package com.zanxiang.game.back.serve.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zanxiang.erp.security.annotation.PreAuthorize;
import com.zanxiang.game.back.serve.pojo.dto.GameTencentOrderDTO;
import com.zanxiang.game.back.serve.pojo.dto.GameTencentUserDTO;
import com.zanxiang.game.back.serve.pojo.vo.GameTencentOrderVO;
import com.zanxiang.game.back.serve.pojo.vo.GameTencentUserVO;
import com.zanxiang.game.back.serve.service.IGameTencentOrderService;
import com.zanxiang.game.back.serve.service.IGameTencentUserService;
import com.zanxiang.module.util.NumberUtil;
import com.zanxiang.module.util.exception.BaseException;
import com.zanxiang.module.util.pojo.ResultVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.util.List;

@RestController
@RequestMapping("/tencent")
@Api("回传策略")
public class TencentLogController {
    @Autowired
    private IGameTencentUserService tencentUserService;
    @Autowired
    private IGameTencentOrderService tencentOrderService;

    @PreAuthorize(permissionKey = "gameBack:tencent:orderLogs")
    @PostMapping("/orderLogs")
    @ApiOperation(value = "腾讯订单回传日志列表")
    public ResultVO<IPage<GameTencentOrderVO>> tencentOrderLogList(@RequestBody GameTencentOrderDTO dto) {
        return ResultVO.ok(tencentOrderService.tencentOrderLogList(dto));
    }

    @PreAuthorize(permissionKey = "gameBack:tencent:orderReport")
    @PostMapping("/orderReport/{ids}/{backMoney}")
    @ApiOperation(value = "腾讯订单手动上报")
    public ResultVO<Boolean> tencentOrderReport(@PathVariable("ids") List<Long> ids, @PathVariable("backMoney") BigDecimal backMoney) {
        if(backMoney.compareTo(BigDecimal.ZERO) <= 0) {
            throw new BaseException("回传金额错误！");
        }
        return ResultVO.ok(tencentOrderService.tencentOrderReport(ids, NumberUtil.multiply100(backMoney).longValue()));
    }

    @PreAuthorize(permissionKey = "gameBack:tencent:userLogs")
    @PostMapping("/userLogs")
    @ApiOperation(value = "腾讯用户回传日志列表")
    public ResultVO<IPage<GameTencentUserVO>> tencentUserLogList(@RequestBody GameTencentUserDTO dto) {
        return ResultVO.ok(tencentUserService.tencentUserLogList(dto));
    }

    @PreAuthorize(permissionKey = "gameBack:tencent:userReport")
    @PostMapping("/userReport/{ids}")
    @ApiOperation(value = "腾讯用户手动上报")
    public ResultVO<Boolean> tencentUserReport(@PathVariable("ids") List<Long> ids) {
        return ResultVO.ok(tencentUserService.tencentUserReport(ids));
    }
}
