package com.zanxiang.game.back.serve.pojo.dto;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.zanxiang.game.back.serve.pojo.vo.GameBackPolicyVO;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 游戏回传策略表
 * </p>
 *
 * @author auto
 * @since 2023-06-01
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
public class GameBackPolicyDTO implements Serializable {

    private static final long serialVersionUID = 1L;

    @NotBlank(message = "回传策略名称不能为空")
    @ApiModelProperty("回传策略名")
    private String backPolicyName;

    @ApiModelProperty("回传类型（1：首单、次单、2：首日、次日）")
    private Integer backUnit;

    /**
     * 首充-小额判定金额（小于等于）
     */
    @NotNull
    @ApiModelProperty("首充-小额判定金额（小于等于）")
    private BigDecimal firstMinMoney;

    /**
     * 首充-小额回传方式（概率/比例）
     */
    @NotNull
    @ApiModelProperty("首充-小额回传方式（概率/比例）")
    private Integer firstMinMoneyType;

    /**
     * 首充-小额回传概率/比例
     */
    @NotBlank
    @ApiModelProperty("首充-小额回传概率/比例")
    private String firstMinMoneyRate;

    /**
     * 首充-大额判定金额（大于等于）
     */
    @NotNull
    @ApiModelProperty("首充-大额判定金额（大于等于）")
    private BigDecimal firstMaxMoney;

    /**
     * 首充-大额回传方式（概率/比例）
     */
    @NotNull
    @ApiModelProperty("首充-大额回传方式（概率/比例）")
    private Integer firstMaxMoneyType;

    /**
     * 首充-大额回传概率/比例
     */
    @NotBlank
    @ApiModelProperty("首充-大额回传概率/比例")
    private String firstMaxMoneyRate;

    /**
     * 首充-其它金额回传方式（概率/比例）
     */
    @NotNull
    @ApiModelProperty("首充-其它金额回传方式（概率/比例）")
    private Integer firstOtherMoneyType;

    /**
     * 首充-其它金额回传概率/比例
     */
    @NotBlank
    @ApiModelProperty("首充-其它金额回传概率/比例")
    private String firstOtherMoneyRate;

    /**
     * 首充-其它金额回传概率/比例
     */
    @ApiModelProperty("首充-降档规则")
    private List<GameBackPolicyVO.PolicyDownLevel> firstDownLevel;
    /**
     * 首充-按用户订单数卡单
     */
    @ApiModelProperty("首充-按用户订单数卡单")
    private Integer firstBackCountOfUser;

    /**
     * 次单-小额判定金额（小于等于）
     */
    @NotNull
    @ApiModelProperty("次单-小额判定金额（小于等于）")
    private BigDecimal rechargeMinMoney;

    /**
     * 次单-小额回传方式（概率/比例）
     */
    @NotNull
    @ApiModelProperty("次单-小额回传方式（概率/比例）")
    private Integer rechargeMinMoneyType;

    /**
     * 次单-小额回传概率/比例
     */
    @NotBlank
    @ApiModelProperty("次单-小额回传概率/比例")
    private String rechargeMinMoneyRate;

    /**
     * 次单-大额判定金额（大于等于）
     */
    @NotNull
    @ApiModelProperty("次单-大额判定金额（大于等于）")
    private BigDecimal rechargeMaxMoney;

    /**
     * 次单-大额回传方式（概率/比例）
     */
    @NotNull
    @ApiModelProperty("次单-大额回传方式（概率/比例）")
    private Integer rechargeMaxMoneyType;

    /**
     * 次单-大额回传概率/比例
     */
    @NotBlank
    @ApiModelProperty("次单-大额回传概率/比例")
    private String rechargeMaxMoneyRate;

    /**
     * 次单-其它金额回传方式（概率/比例）
     */
    @NotNull
    @ApiModelProperty("次单-其它金额回传方式（概率/比例）")
    private Integer rechargeOtherMoneyType;

    /**
     * 次单-其它金额回传概率/比例
     */
    @NotBlank
    @ApiModelProperty("次单-其它金额回传概率/比例")
    private String rechargeOtherMoneyRate;

    @ApiModelProperty("注册充值回传的最大间隔时间（超时不回传）")
    private Long regPayIntervalTime;

    /**
     * 首充-其它金额回传概率/比例
     */
    @ApiModelProperty("次单-降档规则")
    private List<GameBackPolicyVO.PolicyDownLevel> rechargeDownLevel;

    /**
     * 次单-按用户订单数卡单
     */
    @ApiModelProperty("次单-按用户订单数卡单")
    private Integer rechargeBackCountOfUser;

    /**
     * 是否补单
     */
    @ApiModelProperty("是否补单")
    private Boolean markUpOrder;

    /**
     * 补单-小额判定金额（小于等于）
     */
    @ApiModelProperty("补单-小额判定金额（小于等于）")
    private BigDecimal markUpOrderMinMoney;

    /**
     * 补单-小额回传方式（概率/比例）
     */
    @ApiModelProperty("补单-小额回传方式（概率/比例）")
    private Integer markUpOrderMinMoneyType;

    /**
     * 补单-小额回传概率/比例
     */
    @ApiModelProperty("补单-小额回传概率/比例")
    private String markUpOrderMinMoneyRate;

    /**
     * 补单-大额判定金额（大于等于）
     */
    @ApiModelProperty("补单-大额判定金额（大于等于）")
    private BigDecimal markUpOrderMaxMoney;

    /**
     * 补单-大额回传方式（概率/比例）
     */
    @ApiModelProperty("补单-大额回传方式（概率/比例）")
    private Integer markUpOrderMaxMoneyType;

    /**
     * 补单-大额回传概率/比例
     */
    @ApiModelProperty("补单-大额回传概率/比例")
    private String markUpOrderMaxMoneyRate;

    /**
     * 补单-其它金额回传方式（概率/比例）
     */
    @ApiModelProperty("补单-其它金额回传方式（概率/比例）")
    private Integer markUpOrderOtherMoneyType;

    /**
     * 补单-其它金额回传概率/比例
     */
    @ApiModelProperty("补单-其它金额回传概率/比例")
    private String markUpOrderOtherMoneyRate;

    /**
     * 首充-其它金额回传概率/比例
     */
    @ApiModelProperty("补单-降档规则")
    private List<GameBackPolicyVO.PolicyDownLevel> markUpDownLevel;

    /**
     * 补单-按用户订单数卡单
     */
    @ApiModelProperty("补单-按用户订单数卡单")
    private Integer markUpBackCountOfUser;


}
