package com.zanxiang.game.module.sdk.config;

import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.common.serialization.StringSerializer;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.net.InetAddress;
import java.util.Properties;

/**
 * @author : lingfeng
 * @time : 2022-09-28
 * @description : kafka配置
 */
@Slf4j
@Configuration
public class KafkaConfig {

    @Value("${spring.kafka.game-sdk.bootstrap-servers}")
    private String gameSdkKafkaSevers;

    @Bean("gameSdkKafkaProducer")
    public KafkaProducer<String, String> gameKafkaProducer() {
        String clientId = "UNKNOWN";
        try {
            clientId = InetAddress.getLocalHost().getHostAddress();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        Properties props = new Properties();
        props.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, gameSdkKafkaSevers);
        props.put(ProducerConfig.CLIENT_ID_CONFIG, clientId);
        props.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringSerializer.class);
        props.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, StringSerializer.class);
        return new KafkaProducer<>(props);
    }
}
